/*
 * cougar.c
 *
 * This module has the main control program for the Wild Cougar robot
 * 
 *
 */
#include <hc11.h>
#include <mboard.h>
#define IC2_SVC
#define IC1_SVC
#define DOUSERINT
#define USER_INIT
#define WANT_SERVOS
#include <mbintsvc.h>

struct pid {
	int	setpoint;
	int	process;
	int	encoder;
	int	drift;
	int	speed;
	char	direction;
	int	dist;
} left, right;

void
ic1_svc()
{
	right.encoder++;
	right.dist++;
	TFLG1 = 4;
}
void
ic2_svc()
{
	left.encoder++;
	left.dist++;
	TFLG1 = 2;
}

#define SAMPLE_TIME 100
#define INTEGRATE_TIME 4

int	samp_time;		/* Count down (in mS) until next sample */
int	integ_time;		/* Count down in samples for integration */
int	new_samp;		/* Global flag indicating a sample has 
				   occured */
int
do_pid(p, do_drift)
	struct pid *p;
	int	do_drift;
{
	int	delta;

	p->process = p->encoder; 	/* number of ticks accumlated */
	p->encoder = 0;			/* zero the accumulator */

	/*
	 * Process the proportional control. 'delta' is the current
	 * error signal, we multiply it by a constant (.5) and feed
	 * it back into the system. Note that since we subtract the
	 * the setpoint from process it is always _negative_ feed
 	 * back. Motor too fast == -delta, too slow == +delta
	 */
	delta = p->setpoint - p->process; /* our error signal */
	p->speed = p->speed + (delta/2); 

	/* 
	 * Process the integration term, in this case we add up
	 * the last "INTEGRATE_TIME" deltas and check to see 
	 * how close to zero they are. If they are positive we
	 * nudge the motors slower (motor_speed--) if they
	 * are negative we nudge them a bit faster. This is sort
	 * of fine tuning motor speed over time.
	 */
	p->drift = p->drift + delta;
	if (do_drift) {
		if (p->drift > 0)
			p->speed++;	/* integration constant = 1 */
		if (p->drift < 0);
			p->speed--;
		p->drift = 0;
	}

	/*
	 * This code is a basic limiter, it limits the value of
	 * motor speed to a legal value between 0 and 16. If
	 * you were driving the motor backwards you would want
	 * to have a direction variable set to -1 and pass motr
	 * the quantity "dir * motr_speed".
	 */
	if (p->speed < 0)
		p->speed = 0;
	if (p->speed > 16)
		p->speed = 16;
}

extern void do_cmotors();
/*
 * This is called every millsecond by the tick subroutine
 */
void
do_userint()
{
	int	d;

	/* When samp_time counts down to zero, go to work. */
	do_cmotors();
	if (--samp_time == 0) {
		samp_time = SAMPLE_TIME; /* reset the count down timer */
		new_samp++;
		if (--integ_time == 0) {
			d = 1;
			integ_time = INTEGRATE_TIME;
		} else
			d = 0;

		do_pid(&left, d);
		do_pid(&right, d);
		cmotor(2, left.direction * left.speed);
		cmotor(1, right.direction * right.speed);
	}
}

/*
 * Very simple ping function. This function will tell the sonar module
 * to take a reading and return the result.
 */

int
ping()
{
	int	i;

	i = 0;
	PORTD |= 8;	/* take init high */
	/* wait for echo */
	while (((PORTD & 4) == 0) && (i < 2000))
	    i++;
	PORTD &= ~8;
	return i;
}

/*
 * This is the maximum speed of the motor in ticks/sample. It was 
 * determined empirically by setting the setpoint to be 100 and
 * watching the process variable to see how high it got with the
 * motor at maximum.
 */
#define MAX_MOTOR_SPEED 11
extern unsigned char motor_ctl;
extern unsigned int speed[];

void
all_stop() {
	int	i;
	PORTC = 0;
	motor_ctl = 0;
	speed[0] = speed[1] = -1;
	for (i = 0; i < sizeof(left); i++)
		*((char *)(&left) + i) = 0;
	for (i = 0; i < sizeof(right); i++)
		*((char *)(&right) + i) = 0;
}

int
user_init()
{
	DDRD |= 8;
 	DDRC = 0xf0;	/* Port C is 1/2 an output */
	samp_time = SAMPLE_TIME;
	new_samp = 0;
	TFLG1 = 2+4;	/* Clear timer flag */
	TCTL2 = 0x3c;	/* Capture any edge */
	TMSK1 |= 2+4;	/* Enable interrupts from IC2 */
	all_stop();
	return 0;
}
void move(int dir1, int dir2, int s) {
	int x = 0;

	left.direction = dir1;
	right.direction = dir2;
	left.setpoint = right.setpoint = 3;
	while (x < 20) {
	    if ((left.dist >= s) || (right.dist >= s))
		break;
	    x++;
	}
	if (x == 20) {
	    printdec(left.dist);
	    putchar(':');
	    printdec(right.dist);
	    all_stop();
	    while (1) ;
   	}
	all_stop();
}
#define move_forward(s) move(1, 1, s)
#define move_back(s) move(-1, -1, s);
#define turn_left(s) move(-1, 1, s);
#define turn_right(s) move(1, -1, s);

/*
 * This is the main function and it is designed to run the motors at
 * "stop" , 25% speed, 50% speed, 75% speed, and 100% speed. If this
 * were a navy boat that would be slow, cruise, full, and flank speed.
 *
 */

void
main()
{
	int	speed_delta = MAX_MOTOR_SPEED / 4;
	int	i, j, set_speed = 0;
	int	home, mini;

#define TURN_CONSTANT 1200
	all_stop();
	right.direction = left.direction = 1;
	msleep(10000);
	mini = 3000;
	while (1) { 
	    move_forward(20);
	    turn_right(20);
	    move_forward(20);
	    turn_right(20);
	    move_forward(20);
	    turn_right(20);
	    move_forward(20);
	    turn_right(20);
	    move_forward(20);
		while (1) ;
/*
	    move_forward(20);
	    turn_right(TURN_CONSTANT);
	    move_forward(00);
	    turn_left(TURN_CONSTANT);
	    move_forward(1000);
	    turn_left(TURN_CONSTANT);
	    move_forward(1500);
	    turn_left(TURN_CONSTANT);
	    move_forward(1000);
	    turn_right(TURN_CONSTANT);
	    move_forward(1000);
*/
	    while (1) ;
	}
/*
for (i = -90; i < 90; i += 10) {
	        servo(1, i);
	        msleep(100);
	        servo_off(1);
		j = ping();
		if (j < mini) {
		    home = i;
		    mini = j;
	        }
	    }
	    servo(1, home);
	    msleep(150);
	    servo_off(1);
	    mini = 3000;
	    msleep(10000);
	}
*/
}
