/*
 * cougar.c
 *
 * This module has the main control program for the Wild Cougar robot
 * 
 *
 */
#include <hc11.h>
#include <mboard.h>
#define IC2_SVC
#define IC1_SVC
#define DOUSERINT
#define USER_INIT
#include <mbintsvc.h>

struct pid {
	int	setpoint;
	int	process;
	int	encoder;
	int	drift;
	int	speed;
} left, right;

void
ic1_svc()
{
	right.encoder++;
	TFLG1 = 4;
}
void
ic2_svc()
{
	left.encoder++;
	TFLG1 = 2;
}

#define SAMPLE_TIME 100
#define INTEGRATE_TIME 4

int	samp_time;		/* Count down (in mS) until next sample */
int	integ_time;		/* Count down in samples for integration */
int	new_samp;		/* Global flag indicating a sample has 
				   occured */
int
do_pid(p, do_drift)
	struct pid *p;
	int	do_drift;
{
	int	delta;

	p->process = p->encoder; 	/* number of ticks accumlated */
	p->encoder = 0;			/* zero the accumulator */

	/*
	 * Process the proportional control. 'delta' is the current
	 * error signal, we multiply it by a constant (.5) and feed
	 * it back into the system. Note that since we subtract the
	 * the setpoint from process it is always _negative_ feed
 	 * back. Motor too fast == -delta, too slow == +delta
	 */
	delta = p->setpoint - p->process; /* our error signal */
	p->speed = p->speed + (delta/2); 

	/* 
	 * Process the integration term, in this case we add up
	 * the last "INTEGRATE_TIME" deltas and check to see 
	 * how close to zero they are. If they are positive we
	 * nudge the motors slower (motor_speed--) if they
	 * are negative we nudge them a bit faster. This is sort
	 * of fine tuning motor speed over time.
	 */
	p->drift = p->drift + delta;
	if (do_drift) {
		if (p->drift > 0)
			p->speed++;	/* integration constant = 1 */
		if (p->drift < 0);
			p->speed--;
		p->drift = 0;
	}

	/*
	 * This code is a basic limiter, it limits the value of
	 * motor speed to a legal value between 0 and 16. If
	 * you were driving the motor backwards you would want
	 * to have a direction variable set to -1 and pass motr
	 * the quantity "dir * motr_speed".
	 */
	if (p->speed < 0)
		p->speed = 0;
	if (p->speed > 16)
		p->speed = 16;
}

extern void do_cmotors();
/*
 * This is called every millsecond by the tick subroutine
 */
void
do_userint()
{
	int	d;

	/* When samp_time counts down to zero, go to work. */
	do_cmotors();
	if (--samp_time == 0) {
		samp_time = SAMPLE_TIME; /* reset the count down timer */
		new_samp++;
		if (--integ_time == 0) {
			d = 1;
			integ_time = INTEGRATE_TIME;
		} else
			d = 0;

		do_pid(&left, d);
		do_pid(&right, d);
		cmotor(2, left.speed);
		cmotor(1, right.speed);
	}
}

/*
 * This is the maximum speed of the motor in ticks/sample. It was 
 * determined empirically by setting the setpoint to be 100 and
 * watching the process variable to see how high it got with the
 * motor at maximum.
 */
#define MAX_MOTOR_SPEED 11
extern unsigned char motor_ctl;
extern unsigned int speed[];
int
user_init()
{
	int	i;

 	DDRC = 0xf0;	/* Port C is 1/2 an output */
	PORTC = 0;
	motor_ctl = 0;
	speed[0] = speed[1] = -1;
	for (i = 0; i < sizeof(left); i++)
		*((char *)(&left) + i) = 0;
	for (i = 0; i < sizeof(right); i++)
		*((char *)(&right) + i) = 0;
	return 0;
}
/*
 * This is the main function and it is designed to run the motors at
 * "stop" , 25% speed, 50% speed, 75% speed, and 100% speed. If this
 * were a navy boat that would be slow, cruise, full, and flank speed.
 *
 */

void
main()
{
	int	speed_delta = MAX_MOTOR_SPEED / 4;
	int	i, j, set_speed = 0;


	samp_time = SAMPLE_TIME;
	new_samp = 0;
	TFLG1 = 2+4;	/* Clear timer flag */
	TCTL2 = 0x3c;	/* Capture any edge */
	TMSK1 |= 2+4;	/* Enable interrupts from IC2 */
	right.setpoint = left.setpoint = (MAX_MOTOR_SPEED  * set_speed) / 4;
	right.speed = 0;
	left.speed = 0;
	left.setpoint = 0;
	j = 30;
	while (1) {
		if (new_samp) {
			new_samp = 0;
			puts("\rLS: ");
			numout(right.setpoint);
			puts(", LP: ");
			numout(right.process);
			puts(", Lspd: ");
			numout(right.speed);
			puts("\r");
			if (j) {
				j--;
				if (j == 0) 
					left.setpoint = right.setpoint = 0;
			}
				
			/* change the motor speed */
			if (! digital(0)) {
				msleep(100);
				if (digital(0))
					continue;
				msleep(100);
				set_speed = (set_speed + 1) % 5;
				if (set_speed)
					j = 50/set_speed;
				left.setpoint = right.setpoint =
					(MAX_MOTOR_SPEED * set_speed) / 4;
			}
		}
	}
}
